// 極楽画像計測（Gokuraku_measure）用データファイル結合プログラム(Gokuraku_merge)
// Created by Tatsuya Shirai
// National Institute of Technology, Suzuka college
// Mechanical Department
//
// Ver. 1.0 : 2022.06.08
// Ver. 2.0 : テキストファイル名の代わりに画像ファイル名を記録する（パス無し）

// 参考URL（ドラッグ＆ドロップに関して
// https://qiita.com/rain-lotus/items/258fa3defd41846bfd50
// https://sites.google.com/site/gutugutu30/other/processingdedoragguandodoroppu

Data         data;
ImportFile   im_file;
ExportFile   ex_file;
ShortMessage shortMessage;

final int Scr_w = 800;
final int Scr_h = 380;

void setup() {
  data         = new Data();
  im_file      = new ImportFile();
  ex_file      = new ExportFile();
  shortMessage = new ShortMessage();

  //サイズ変更を許可する
  surface.setResizable(true);

  drop_init();
}

void draw(){
  int x  = 16;
  int y  = 40;
  int dx = 100;
  int dy = 40;

  // 画面消去
  surface.setSize(Scr_w, Scr_h);
  fill(72, 72, 72);
  rect(0, 0, Scr_w - 1, Scr_h - 1);

  fill( 80, 255, 80);
  // タイトル表示
  textSize(28);
  text("Gokuraku merge (@tatsuva) Ver." + Version, x, y); y += dy;

  // メニュー表示
  textSize(24);
  fill(255, 80, 80);
  text("[I]",    x, y); text(": import data file",     x + dx, y);  y += dy;
  text("[F]",    x, y); text(": select export folder", x + dx, y);  y += dy;
  text("[C]",    x, y); text(": clear data",           x + dx, y);  y += dy;
  text("[E]",    x, y); text(": export merged data",   x + dx, y);  y += dy;
  text("[ESC]",  x, y); text(": quit",                 x + dx, y);  y += dy;
  y += 16;
  textSize(16);
  fill(255, 255, 255);
  if (ex_file.export_folder == null) text("Folder: not selected", x, y);             
  else                               text("Folder: " + ex_file.export_folder, x, y);
  y += 20;
  if (data.num > 0)                  text("Imported " + data.num + " files", x, y);
  else                               text("Imported no files", x, y);
  y += dy;
  shortMessage.display_short_message(x, y);                        y += dy;
  
}

void mousePressed()
{
  if (mouseButton == RIGHT) open_fileselector();
}

void keyPressed()
{
  switch (keyCode) {
    case 'I' :
      open_fileselector();
      break;
    case 'F' :
      // エクスポートするフォルダーの選択
      File fp;
      fp = new File(ex_file.export_folder);
      selectFolder("Select a folder to process:", "folderSelected", fp);
      break;
    case 'C' : 
      // 全データ消去
      data.allclear();
      break;
    case 'E' : 
      // 全データのファイルへの出力
      ex_file.export_alldata(data);
      break;
  }
}

// ファイル選択ダイアログボックスの呼び出し
void open_fileselector()
{
  selectInput("Select a file to process:", "fileSelected");
}
// ファイル選択後の処理
void fileSelected(File selection) 
{
  if (selection == null) return;
  im_file.read(selection.getAbsolutePath(), data);
}

// エクスポートするフォルダーの選択
void folderSelected(File selection) {
  if (selection == null) return;  // キャンセル時
  ex_file.set_export_folder(selection.getAbsolutePath());
  shortMessage.set_message("User select folder = " + ex_file.export_folder);
}

//// ドラッグ＆ドロップ関係
// 上記URL参照
// ドロップされたファイルの処理
void fileSelected(List<File> fs) {
  for(File f:fs){
    // ドラッグ＆ドロップされたリストの中にフォルダーが含まれていた場合は，その層までは自動的に読み込む
    if (f.isDirectory()) {
      for (File sf:f.listFiles()) {
        if (sf.isFile()) im_file.read(sf.getAbsolutePath(), data);
      }
    } else im_file.read(f.getAbsolutePath(), data);
  }
}

import java.awt.datatransfer.*;  
import java.awt.dnd.*;  
import java.io.File;  
import java.io.IOException;  
import java.awt.Component;
import java.util.List;

DropTarget dropTarget;
Component  component;

void drop_init() {  
  component = (Component)this.surface.getNative();
  dropTarget = new DropTarget(component, new DropTargetListener() {
    public void dragEnter(DropTargetDragEvent dtde) {
    }  
    public void dragOver(DropTargetDragEvent dtde) {
    }  
    public void dropActionChanged(DropTargetDragEvent dtde) {
    }  
    public void dragExit(DropTargetEvent dte) {
    }  
    public void drop(DropTargetDropEvent dtde) {  
      dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);  
      Transferable trans = dtde.getTransferable();  
      List<File> fileNameList = null;  
      if (trans.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {  
        try {  
          fileNameList = (List<File>)  
            trans.getTransferData(DataFlavor.javaFileListFlavor);
        } 
        catch (UnsupportedFlavorException ex) {
        } 
        catch (IOException ex) {
        }
      }  
      if (fileNameList == null) return;  
      fileSelected(fileNameList);
    }
  }
  );
}
