// 極楽画像計測（Gokuraku_measure）用可視化プログラム(Gokuraku_visualizer)
// Created by Tatsuya Shirai
// National Institute of Technology, Suzuka college
// Mechanical Department
//
// Ver. 1.0 : 2022.06.08


Data         data;
CurrentImage ci;
ImportFile   im_file;
ExportFile   ex_file;
ShortMessage shortMessage;
InputInteger inputInteger;
KeyStatus    keyStatus;

final int Scr_w = 800;
final int Scr_h = 550;

enum Menu_status {main, setting, sizeinputing, delayinputing, linewidthinputing};
Menu_status menu_status;

void setup() {
  data         = new Data();
  im_file      = new ImportFile();
  ci           = new CurrentImage();
  ex_file      = new ExportFile();
  shortMessage = new ShortMessage();
  inputInteger = new InputInteger();
  keyStatus    = new KeyStatus();
  menu_status  = Menu_status.main;

  //サイズ変更を許可する
  surface.setResizable(true);

  drop_init();
}

void draw(){
  int x  = 8;
  int y  = 48;
  int dx = 64;
  int dy = 40;

  noStroke();
  if (ci.preview.playing != PreviewMode.stop) {
    ci.play(data);
    y = height - 16;
  } else {
    // 画面消去
    surface.setSize(Scr_w, Scr_h);
//  fill(72, 72, 72);
    background(72, 72, 72);
  
    fill(180, 180, 255);
    // タイトル表示
    textSize(28);
    text("Gokuraku visualizer (@tatsuva) Ver." + Version, x, y); y += dy;
  
    // メニュー表示
    textSize(18);
    text("The data file should be sorted in advance using Excel or other tools.",  x, y); y += dy;
    textSize(24);
    fill(ci.plotMark.red(), ci.plotMark.green(), ci.plotMark.blue());
    if (menu_status == Menu_status.sizeinputing) {
      noStroke();
      textSize(24);
      text("Plot size = " + inputInteger.value, x, y); y += dy;
    } else if (menu_status == Menu_status.delayinputing) {
      noStroke();
      textSize(24);
      text("Delay per image = " + inputInteger.value, x, y); y += dy;
    } else if (menu_status == Menu_status.linewidthinputing) {
      noStroke();
      textSize(24);
      text("Line width = " + inputInteger.value, x, y); y += dy;
    } else if (menu_status == Menu_status.main) {
      text("[I]",   x, y); text(": import merged data file",   x + dx, y); y += dy;
      text("[S]",   x, y); text(": setting submenu",           x + dx, y); y += dy;
      text("[D]",   x, y); text(": show data list(console)",   x + dx, y); y += dy;
      text("[L]",   x, y); text(": import define_stick (" + 
                                (ci.preview.define_stick.length == 0 ? "none" : "defined") + ")",     
                                                               x + dx, y); y += dy;
      text("[P]",   x, y); text(": preview images (ESC:halt / SPC, <-, ->:pause)",
                                                               x + dx, y); y += dy;
      text("[F]",   x, y); text(": select folder",             x + dx, y); y += dy;
      text("[E]",   x, y); text(": export image files",        x + dx, y); y += dy;
      text("[ESC]", x, y); text(": quit",                      x + dx, y); y += dy;
      y += 16;
  
      textSize(16);
      fill(255, 255, 255);
      text("Path format: " + ex_file.fullpathname, x, y);              y += 20;
      if (data.num > 0) {
                       text("Imported " + data.num + " files", x, y); y += dy;
      } else {
                       text("Imported no data", x, y);                y += dy;
      }
    } else if (menu_status == Menu_status.setting) {
      text("[B]", x, y); text(": change background color (=" + color2str(BG_color[ci.bg_color][0], BG_color[ci.bg_color][1], BG_color[ci.bg_color][2]) + ")", x + dx, y); y += dy;
      text("[T]", x, y); text(": change plot type  (=" + ci.plotMark.shapeStr() + ")",  x + dx, y); y += dy;
      text("[C]", x, y); text(": change plot color (=" + ci.plotMark.color_str() + ")", x + dx, y); y += dy;
      text("[P]", x, y); text(": change plot size  (=" + ci.plotMark.size + ")",        x + dx, y); y += dy;
      text("[W]", x, y); text(": change line width (=" + ci.plotMark.lineWidth + ")",   x + dx, y); y += dy;
      text("[I]", x, y); text(": toggle enable image flag  (" + (ci.preview.enable_image ? "true" : "false") + ")",  
                                                              x + dx, y); y += dy; 
      text("[S]", x, y); text(": toggle enable scale flag  (" + (ci.preview.enable_scale ? "true" : "false") + ")",  
                                                           x + dx, y); y += dy; 
      text("[L]", x, y); text(": toggle line segment flag (" + (ci.preview.enable_stick ? "true" : "false") + ")",  
                                                           x + dx, y); y += dy; 
      text("[D]", x, y); text(": change delay per image (=" + ci.preview.delay_per_image + "[ms], > 1)", 
                                                           x + dx, y); y += dy;
      text("[Q]", x, y); text(": quit sub menu",           x + dx, y); y += dy;
    }
  }
  shortMessage.display_short_message(x, y);
  y += 20;
}

void mousePressed()
{
  if (mouseButton == RIGHT) selectInput("Select a file to process:", "fileSelected");
}

void keyReleased()
{
  keyStatus.release(keyCode);
}

void keyPressed()
{
  keyStatus.push(keyCode);
  if (menu_status == Menu_status.sizeinputing) {
    // plot_size を修正時
    if (inputInteger.inputing_now) {
      if (key == ENTER) {
        int num = inputInteger.value;
        if (num > 0) ci.plotMark.size = num;
        inputInteger.end();
        menu_status = Menu_status.setting;
        shortMessage.set_message("Change plot size to " + ci.plotMark.size);
      } else if (key == ESC) {
        menu_status = Menu_status.setting;
        shortMessage.set_message("Canceled");
        key = 0;
      } else inputInteger.keyin(key);
      return;
    }
  } else if (menu_status == Menu_status.delayinputing) {
    // delay_per_image を修正時
    if (inputInteger.inputing_now) {
      if (key == ENTER) {
        int num = inputInteger.value;
        if (num > 1) ci.preview.delay_per_image = num;
        inputInteger.end();
        menu_status = Menu_status.setting;
        shortMessage.set_message("Change delay per image to " + ci.preview.delay_per_image + "[ms]");
      } else if (key == ESC) {
        menu_status = Menu_status.setting;
        shortMessage.set_message("Canceled");
        key = 0;
      } else inputInteger.keyin(key);
      return;
    }
  } else if (menu_status == Menu_status.linewidthinputing) {
    // lineWidth を修正時
    if (inputInteger.inputing_now) {
      if (key == ENTER) {
        int num = inputInteger.value;
        if (num >= 1) ci.plotMark.lineWidth = num;
        inputInteger.end();
        menu_status = Menu_status.setting;
        shortMessage.set_message("Change line width to " + ci.plotMark.lineWidth);
      } else if (key == ESC) {
        menu_status = Menu_status.setting;
        shortMessage.set_message("Canceled");
        key = 0;
      } else inputInteger.keyin(key);
      return;
    }
  } else if (menu_status == Menu_status.main) {
    switch (keyCode) {
      case ESC :
        if (ci.preview.playing != PreviewMode.stop) {
          if (ci.preview.export_enable) {
            shortMessage.set_message("Halt: exported image file : " + (ci.preview.count) + " files");
          }
          ci.preview.stop_preview();
          key = 0;
        }
        break;
      case RIGHT :  // プレビューを一時停止中にコマを移動（進む）
        if ((ci.preview.playing == PreviewMode.pause) && (ci.preview.export_enable == false)) {
          if (keyStatus.shift )    ci.preview.last_image();    // SHIFT同時押しで最終コマへ
          else if (keyStatus.ctrl) ci.preview.next_image(10);  // CTRL同時押しで10コマ先に
          else                     ci.preview.next_image();
        }
        break;
      case LEFT :   // プレビューを一時停止中にコマを移動（戻る）
        if ((ci.preview.playing == PreviewMode.pause) && (ci.preview.export_enable == false)) {
          if (keyStatus.shift)     ci.preview.first_image();   // SHIFT同時押しで最初のコマへ
          else if (keyStatus.ctrl) ci.preview.prev_image(10);  // CTRL同時押しで10コマ前に
          else                     ci.preview.prev_image();
        }
        break;
      case ' ' :
        if (ci.preview.playing == PreviewMode.playing) {       // 再生中ならば一時停止
          ci.preview.playing = PreviewMode.pause;
        } else if (ci.preview.playing == PreviewMode.pause) {  // 一時停止中ならば再生
          ci.preview.playing = PreviewMode.playing;
        }
        break;
      case 'I' : 
        // 結合されたデータファイルの読み込み
        selectInput("Select a file to process:", "fileSelected");
        break;
      case 'S' :
        // 設定のサブメニューに移動
        menu_status = Menu_status.setting;
        break;
      case 'D' :
        data.show_data_list();
        break;
      case 'L' :
        selectInput("Select a define stick:", "stickSelected");
        break;
      case 'P' :
        ci.preview.start_preview();
        break;
      case 'F' :
        // エクスポートするフォルダーの選択
        File fp;
        fp = new File(ex_file.export_folder);
        selectFolder("Select a folder to process:", "folderSelected", fp);
        break;
      case 'E' : 
        // 画像ファイルの出力
        ex_file.start_export();
        break;
    }
  } else if (menu_status == Menu_status.setting) {
    switch (keyCode) {
      case 'B' :
        ci.change_bg();
        shortMessage.set_message("Change background color to " + color2str(BG_color[ci.bg_color][0], BG_color[ci.bg_color][1], BG_color[ci.bg_color][2]));
        break;
      case 'T' :
        ci.plotMark.change_shape();
        shortMessage.set_message("Change plot type to " + ci.plotMark.shapeStr());
        break;
      case 'C' :
        ci.plotMark.change_color();
        shortMessage.set_message("Change plot color to " + ci.plotMark.color_str());
        break;
      case 'P' :
        inputInteger.start(ci.plotMark.size);
        menu_status = Menu_status.sizeinputing;
        break;
      case 'W' :
        inputInteger.start(ci.plotMark.lineWidth);
        menu_status = Menu_status.linewidthinputing;
        break;
      case 'I' :
        ci.preview.switch_enable_image();
        shortMessage.set_message("Toggled enable image : " + (ci.preview.enable_image ? "true" : "false"));
        break;
      case 'S' :
        ci.preview.switch_enable_scale();
        shortMessage.set_message("Toggled enable scale : " + (ci.preview.enable_scale ? "true" : "false"));
        break;
      case 'L' :
        ci.preview.switch_enable_stick();
        shortMessage.set_message("Toggled enable line segment : " + (ci.preview.enable_stick ? "true" : "false"));
        break;
      case 'D' :
        inputInteger.start(ci.preview.delay_per_image);
        menu_status = Menu_status.delayinputing;
        break;
      case 'Q' :
        menu_status = Menu_status.main;
        break;
    }
    key = 0;
  }
}

// 結合ファイル選択後の処理
void fileSelected(File selection) 
{
  if (selection == null) return;
  if (! im_file.read(selection.getAbsolutePath())) data.allclear();
} //<>//
// Stick定義ファイル選択後の処理
void stickSelected(File selection) 
{
  if (selection == null) return;
  ci.preview.readDefineStick(selection.getAbsolutePath());
}

// エクスポートするフォルダーの選択
void folderSelected(File selection) {
  if (selection == null) return;  // キャンセル時
  ex_file.set_export_information(selection.getAbsolutePath());
  shortMessage.set_message("User select folder = " + ex_file.export_folder);
}

//// ドラッグ＆ドロップ関係
// 上記URL参照
// ドロップされたファイルの処理
void fileSelected(List<File> fs) {
  for(File f:fs){
    // ドラッグ＆ドロップされたリストの中にフォルダーが含まれていた場合は，その層までは自動的に読み込む
    if (f.isDirectory()) {
      for (File sf:f.listFiles()) {
        if (sf.isFile()) { //<>//
          if (! im_file.read(sf.getAbsolutePath())) data.allclear();
          if (data.num > 0) return;
        }
      }
    } else {
      if (! im_file.read(f.getAbsolutePath())) data.allclear(); //<>//
      if (data.num > 0) return;
    }
  }
}

import java.awt.datatransfer.*;  
import java.awt.dnd.*;  
import java.io.File;  
import java.io.IOException;  
import java.awt.Component;
import java.util.List;

DropTarget dropTarget;
Component  component;

void drop_init() {  
  component = (Component)this.surface.getNative();
  dropTarget = new DropTarget(component, new DropTargetListener() {
    public void dragEnter(DropTargetDragEvent dtde) {
    }  
    public void dragOver(DropTargetDragEvent dtde) {
    }  
    public void dropActionChanged(DropTargetDragEvent dtde) {
    }  
    public void dragExit(DropTargetEvent dte) {
    }  
    public void drop(DropTargetDropEvent dtde) {  
      dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);  
      Transferable trans = dtde.getTransferable();  
      List<File> fileNameList = null;  
      if (trans.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {  
        try {  
          fileNameList = (List<File>)  
            trans.getTransferData(DataFlavor.javaFileListFlavor);
        } 
        catch (UnsupportedFlavorException ex) {
        } 
        catch (IOException ex) {
        }
      }  
      if (fileNameList == null) return;  
      fileSelected(fileNameList);
    }
  }
  );
}
